from fhirclient.models.operationoutcome import OperationOutcome
from fhirclient.models.coding import Coding
from inspqcommun.fhir.visitors.base import BaseVisitor

class OperationOutcomeVisitor(BaseVisitor):
    __message_code_url = 'http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/extension/#operationoutcome/messagecode'
    def __init__(self, fhir_resource=None) -> None:

        self.setFhirResource(fhir_resource if fhir_resource else OperationOutcome())

    def getFhirResource(self) -> OperationOutcome:
        return super().getFhirResource()

    def get_issue_count(self) -> int:
        return len(self.getFhirResource().issue)

    def get_issue_code(self, index: int=0) -> str:
        return self.getFhirResource().issue[index].code

    def get_issue_severity(self, index: int=0):
        return self.getFhirResource().issue[index].severity

    def get_issue_details(self, index: int=0) -> Coding:
        details = self._get_coding_par_system(self.getFhirResource().issue[index].details)
        return details if details else self.getFhirResource().issue[index].details[0]
    
    def get_issue_message_code(self, index: int=0) -> str: 
        for ext in self.getFhirResource().issue[index].extension:
            if ext.url == self.__message_code_url:
                return ext.valueString
        return None
    
    def get_issue_diagnostics(self, index: int=0) -> str:
        return self.getFhirResource().issue[index].diagnostics


import os
import requests
import logging
import json
import base64
import io
import pandas

from zipfile import ZipFile, ZIP_DEFLATED
from requests import Response
from requests.exceptions import RequestException
from pandas.errors import ParserError
from pathlib import Path
from inspqcommun.identity.keycloak_tools import KeycloakEnvironment

class PefiClient():
    __base_headers = {
        "Content-type": "application/json"
    }
    __pefi_base_uri = "/PEFI"
    __kc_auth_client_id = None
    def __init__(self, pefi_auth_client_id:str=None, logger=None):
        self.__pefi_url = os.environ.get('PEFI_URL')
        self.__env = os.environ.get('ENV') if 'ENV' in os.environ else 'LOCAL'
        self.__pefi_client = os.environ.get('PEFI_CLIENT_ID') if 'PEFI_CLIENT_ID' in os.environ else "sipmiportablepefiservice{env}".format(env=self.__env)
        if "PEFI_AUTH_CLIENT_ID" in os.environ:
            self.__kc_auth_client_id = os.environ.get("PEFI_AUTH_CLIENT_ID")
        elif pefi_auth_client_id is not None:
            self.__kc_auth_client_id = pefi_auth_client_id
        else:
            self.__kc_auth_client_id = "appmas{env}".format(env=self.__env.lower())
        self.__kc_env = KeycloakEnvironment(defaultAuthClientId=self.__kc_auth_client_id)
        self.__client_roles = {
            self.__pefi_client: {
                'roles': [
                    "connection-service-pefi"
                ]
            }
        }
        self.__logger = logger if logger is not None else logging.Logger()


    def __login(self):
        self.__kc_headers = self.__kc_env.authenticate(
            client_roles=self.__client_roles,
            given_name="Usager",
            family_name="Pefi",
            client_id=self.__kc_auth_client_id)
        self.__headers = self.__base_headers | self.__kc_headers
        
    def __get_pefi(self, uri:str, parameters:dict={}) -> Response:
        try:
            self.__login()
            if self.__pefi_url is None:
                self.__logger.error("L'URL du PEFI n'a pas été défini. Le définir avec la variable d'environnement PEFI_URL")
            url = "{base_url}{base_uri}".format(base_url=self.__pefi_url,
                                                base_uri=uri)
            response = requests.get(url=url,
                                    headers=self.__headers,
                                    params=parameters)
            
            return response
        except Exception as e:
            self.__logger.error(e)
            raise e
    
    def get_nom_fichiers_par_type_et_source(self, type:str, source:str) -> list[str]:
        params = {
            "typeChargement": type,
            "sourceChargement": source
        }
        response = self.__get_pefi(uri="/PEFI/chargements/names", parameters=params)
        self.__logger.info("Appel PEFI code {code}".format(code=response.status_code))
        if response.status_code >= 400:
            self.__logger.error("Erreur {code} lors de l'appel au PEFI: {erreur}".format(
                code=response.status_code,
                erreur=response.reason
            ))
        elif response.status_code == 200:
            self.__logger.info("Réponse du PEFI: {}".format(response.content.decode()))
            liste_fichiers = json.loads(response.content)
            return liste_fichiers
        

    def get_statut_fichier(self,
                           type:str,
                           source:str,
                           fichier:str) -> Response:
        
        uri = "/PEFI/chargements/names/{nom_fichier}/status".format(nom_fichier=fichier)
        params = {
            "typeChargement": type,
            "sourceChargement": source
        }
        response = self.__get_pefi(uri=uri, parameters=params)
        return response
    
    def telecharger_fichiers_par_type_et_source(self,
                                                type:str,
                                                source:str,
                                                fichier:str='latest') -> list[dict]:
        liste_fichiers = []
        try:
            if fichier == 'latest':
                uri = "/PEFI/chargements/latest"
            else:
                uri = "/PEFI/chargements/names/{nom_fichier}".format(nom_fichier=fichier)
            params = {
                "typeChargement": type,
                "sourceChargement": source
            }
            response = self.__get_pefi(uri=uri,parameters=params)
            self.__logger.info("Appel PEFI code {code}".format(code=response.status_code))
            if response.status_code >= 400:
                self.__logger.error("Erreur {code} lors de l'appel au PEFI: {erreur}".format(
                    code=response.status_code,
                    erreur=response.reason
                ))
                
            elif response.status_code == 200:
                # zip_file = base64.b64decode(response.content)
                zip_file = response.content
                with open("/tmp/{type}_{source}_{fichier}.zip".format(type=type, source=source, fichier=fichier), 'wb') as f:
                    f.write(zip_file)
                    f.close()

                z = ZipFile(io.BytesIO(zip_file))

                for file_info in z.filelist:
                    self.__logger.info(file_info.filename)
                    data_file = z.read(file_info.filename)
                    with open("/tmp/{}".format(file_info.filename),'w+') as f:
                        f.write(data_file.decode("iso-8859-1"))
                        f.close
                        fichier_donnees = {}
                        fichier_donnees["nom"] = file_info.filename
                        fichier_donnees["data"] = pandas.read_csv(
                            io.BytesIO(data_file),
                            sep=';',
                            header=None,
                            skiprows=0,
                            encoding='iso-8859-1')
                        liste_fichiers.append(fichier_donnees)
        except ParserError as e:
            self.__logger.error(e)
            raise e
        
        return liste_fichiers
        
    def deposer_fichiers(self,
                        type:str,
                        source:str,
                        nom:str,
                        source_dir:str,
                        zip:bool=True) -> Response:
        uri = "/chargements/{nom_fichier}".format(nom_fichier=nom)
        params = {
                "typeChargement": type,
                "sourceChargement": source
            }
        try:
            fichier = None
            if zip:
                fichier = self.__zip_directory_in_memory(source_dir=source_dir)
            else:
                fichier = self.__read_first_file_in_directory(source_dir=source_dir)

            if fichier is not None:
                self.__login()
                url = f"{self.__pefi_url}{self.__pefi_base_uri}{uri}"
                response = requests.post(url=url,
                                        headers=self.__headers,
                                        params=params,
                                        data=fichier)
                return response
            else:
                self.__logger.error("Le fichier a déposer dans PEFI n'a pas pu être lu.")
                return None
        except RequestException as e:
            self.__logger.error(e)


    def __zip_directory_in_memory(self, source_dir:str) -> bytes:
        zip_buffer = io.BytesIO()
        with ZipFile(zip_buffer, 'w', ZIP_DEFLATED) as zipf:
            for foldername, subfolders, filenames in os.walk(source_dir):
                for filename in filenames:
                    filepath = os.path.join(foldername, filename)
                    arcname = os.path.relpath(filepath, source_dir)
                    with open(filepath, 'rb') as f:
                        zipf.writestr(arcname, f.read())
        zip_buffer.seek(0)
        encoded = base64.b64encode(zip_buffer.read())
        return encoded

    def __read_first_file_in_directory(self, source_dir:str) -> bytes:
        for foldername, subfolders, filenames in os.walk(source_dir):
            for filename in filenames:
                filepath = os.path.join(foldername, filename)
                with open(filepath, 'rb') as f:
                    encoded = base64.b64encode(f.read())
                return encoded
        return None




from inspqcommun.fhir.visitors.base import BaseVisitor
from fhirclient.models.flag import Flag
from fhirclient.models.reference import Reference
from fhirclient.models.coding import Coding
from fhirclient.models.codeableconcept import CodeableConcept
from fhirclient.models.period import Period
from fhirclient.models.fhirdate import FHIRDate
from dateutil.parser import parse

class FlagVisitor(BaseVisitor):
    def __init__(self, fhir_resource: Flag = None) -> None:
        self.setFhirResource(fhir_resource if fhir_resource else Flag())

    def getFhirResource(self) -> Flag:
        return super().getFhirResource()
    
    def get_id(self) -> int:
        return self.getFhirResource().id
    
    def set_id(self, id: str) -> None:
        self.getFhirResource().id = id 

    def set_subject_id(self, id: str) -> None:
        reference = Reference()
        reference.reference = id
        self.getFhirResource().subject = reference

    def get_subject_id(self) -> str:
        return self.getFhirResource().subject.reference
    
    def get_category(self) -> Coding:
        if self.getFhirResource().category:
            for coding in self.getFhirResource().category.coding:
                if coding.system == self.DEFAULT_CODING_SYSTEM and coding.version == self.DEFAULT_CODING_VERSION:
                    return coding
        return None
    
    def set_category(self, coding: Coding) -> None:
        if coding.system is None:
            coding.system = self.DEFAULT_CODING_SYSTEM
        if coding.version is None:
            coding.version = self.DEFAULT_CODING_VERSION
        self.getFhirResource().category = CodeableConcept()
        self.getFhirResource().category.coding = [coding]

    def get_period(self) -> Period:
        return self.getFhirResource().period
    
    def set_period(self, period: Period) -> None:
        self.getFhirResource().period = period

    def set_period_start_date(self, start_date: FHIRDate) -> None:
        period = self.get_period()
        if period is None:
            period = Period()
        period.start = start_date
        self.set_period(period)

    def set_period_start_date_from_str(self, start_date: str) -> None:
        period_start_date = FHIRDate()
        period_start_date.date = parse(start_date)
        self.set_period_start_date(start_date=period_start_date)

    def set_period_end_date(self, end_date: FHIRDate) -> None:
        period = self.get_period()
        if period is None:
            period = Period()
        period.end = end_date
        self.set_period(period)

    def set_period_end_date_from_str(self, end_date: str) -> None:
        period_end_date = FHIRDate()
        period_end_date.date = parse(end_date)
        self.set_period_end_date(end_date=period_end_date)

    def get_code(self) -> Coding:
        if self.getFhirResource().code:
            for coding in self.getFhirResource().code.coding:
                if coding.system == self.DEFAULT_CODING_SYSTEM and coding.version == self.DEFAULT_CODING_VERSION:
                    return coding
        return None
    
    def set_code(self, coding: Coding) -> None:
        if coding.system is None:
            coding.system = self.DEFAULT_CODING_SYSTEM
        if coding.version is None:
            coding.version = self.DEFAULT_CODING_VERSION
        self.getFhirResource().code = CodeableConcept()
        self.getFhirResource().code.coding = [coding]

    


from inspqcommun.fhir.clients.base_client import BaseClient
from inspqcommun.fhir.visitors.bundle import BundleVisitor, Bundle
from fhirclient.models.flag import Flag
from requests import Response
import requests
import json
import logging

log = logging.getLogger(__name__)

class FlagClient(BaseClient):
    flag_endpoiunt = "{base_url}/Flag"
    flag_by_id_endpoint = flag_endpoiunt + "/{id}"

    def __init__(self, base_url=None, base_uri=None, token_header=None, validate_certs=True) -> None:
        super().__init__(base_url=base_url, base_uri=base_uri, token_header=token_header)
        self.validate_certs = validate_certs

    def create(self, flag: Flag) -> Response:
        data = json.dumps(flag.as_json())
        response = requests.post(
            url=self.flag_endpoiunt.format(base_url=self.get_fhir_url()),
            data=data,
            headers=self.headers,
            verify=self.validate_certs
        )
        log.info("POST Flag : {}".format(response.status_code))
        return response

    def get_by_id(self, flag_id: int) -> Response:
        response = requests.get(
            url=self.flag_by_id_endpoint.format(base_url=self.get_fhir_url(), id=flag_id),
            headers=self.headers,
            verify=self.validate_certs
        )
        log.info("GET Flag : {}".format(response.status_code))
        return response

    def update(self, flag: Flag) -> Response:
        data = json.dumps(flag.as_json())
        response = requests.put(
            url=self.flag_by_id_endpoint.format(base_url=self.get_fhir_url(), id=flag.id),
            data=data,
            headers=self.headers,
            verify=self.validate_certs
        )
        log.info("PUT Flag : {}".format(response.status_code))
        return response

    def delete(self, flag_id: int) -> Response:
        response = requests.delete(
            url=self.flag_by_id_endpoint.format(base_url=self.get_fhir_url(), id=flag_id),
            headers=self.headers,
            verify=self.validate_certs
        )
        log.info("DELETE Flag : {}".format(response.status_code))  
        return response
    
    def extract_flag_from_response(self, response: Response) -> Flag:
        content = response.json()
        if 'resourceType' in content and content['resourceType'] == 'Flag':
            return Flag(jsondict=content)
        if content['resourceType'] == 'Bundle':
            bundle = BundleVisitor(fhir_resource=Bundle(jsondict=content))
            flags = bundle.get_entries(recurse=True, resource_type='Flag')
            if len(flags) > 0:
                return Flag(jsondict=flags[0])
        return None


from inspqcommun.identity.keycloak_tools import KeycloakEnvironment
from requests import Response, get
import os
import socket
class UtilitaireNAMClient():
    __kcenv = None
    __headers = {}
    __unam_default_port = "14101"
    __generer_uri = "/generer"
    __valider_uri = "/valider"
    __info_uri = "/info"
    __province = "QC"
    __description_sexe = {
        "M": "MASCULIN",
        "F": "FEMININ"
    }
    __unam_base_url = None

    def __init__(self, unam_base_url:str=None):
        if 'UNAM_BASE_URL' in os.environ:
            self.__unam_base_url = f"{os.environ.get('UNAM_BASE_URL')}/nam"
        elif unam_base_url is not None:
            self.__unam_base_url = f"{unam_base_url}/nam"
        else:
            self.__unam_base_url = "http://{0}:{1}/nam".format(socket.getfqdn(), self.__unam_default_port)
        self.__kcenv = KeycloakEnvironment()
        if self.__kcenv.keycloak_enabled:
            if self.__kcenv.keycloak_auth_user is not None and self.__kcenv.keycloak_auth_password is not None:
                self.__headers = self.__kcenv.__authenticate_by_username_password()
            else:
                self.__headers = self.__kcenv.authenticateByServiceAccount()
    def set_province(self, province:str) -> None:
        self.__province = province

    def get_province(self) -> str:
        return self.__province
    
    def generer(self, nom:str, prenom:str, date_naissance:str, sexe:str) -> Response:
        params = {
            "prenom": prenom,
            "nom": nom,
            "datenaissance": date_naissance,
            "sexe": sexe
        }
        rep = get(url=self.__unam_base_url+self.__generer_uri,
                  params=params,
                  headers=self.__headers)
        return rep
    
    def extraire_nam_generes_from_response(self, response:Response) -> list[str]:
        liste_nam = []
        if response.status_code == 200:
            liste_nam = response.json()
        return liste_nam
    
    def generer_une_liste_de_nam_pour_un_usager(self, nom:str, prenom:str, date_naissance:str, sexe:str) -> list[str]:
        return self.extraire_nam_generes_from_response(self.generer(nom=nom,
                                                                    prenom=prenom,
                                                                    date_naissance=date_naissance,
                                                                    sexe=sexe))
    
    def valider(self, nam:str, province:str = None) -> Response:
        params = {
            "nam": nam,
            "province": province if province is not None else self.get_province()
        }
        rep = get(url=self.__unam_base_url+self.__valider_uri,
                  params=params,
                  headers=self.__headers)
        return rep
    
    def extraire_validite_from_response(self, response:Response) -> bool:
        if response.status_code == 200:
            resultat = response.json()
        else:
            resultat = None
        return resultat
    
    def is_nam_valide(self, nam:str, province:str=None) -> bool:
        return self.extraire_validite_from_response(self.valider(nam=nam,
                                                                province=province))

    def info(self, nam:str) -> Response:
        params = {
            "nam": nam
        }
        rep = get(url=self.__unam_base_url+self.__info_uri,
                  params=params,
                  headers=self.__headers)
        return rep
    
    def extraire_info_from_response(self, response:Response) -> dict:
        if response.status_code == 200:
            return response.json()
        else:
            return {}

    def obtenir_les_info_pour_un_nam(self, nam:str) -> dict:
        return self.extraire_info_from_response(self.info(nam=nam))

    def get_description_sexe(self, code:str) -> str:
        return self.__description_sexe.get(code)


from fhirclient.models.condition import Condition, ConditionEvidence
from fhirclient.models.extension import Extension
from fhirclient.models.coding import Coding
from fhirclient.models.codeableconcept import CodeableConcept
from fhirclient.models.reference import Reference
from fhirclient.models.period import Period
from fhirclient.models.fhirdate import FHIRDate


from inspqcommun.fhir.visitors.base import BaseVisitor

class ConditionVisitor(BaseVisitor):
    LOCATION_URL = 'http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/extensions/#condition/location'
    AGENT_URL = "http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/extensions/#condition/agent"
    ANTIGEN_URL = "http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/extensions/#condition/antigen"
    OTHER_REASON_URL = "http://www.santepublique.rtss.qc.ca/sipmi/fa/1.0.0/extensions/#condition/otherreason"

    def __init__(self, fhir_resource=None) -> None:
        self.setFhirResource(fhir_resource if fhir_resource else Condition())
    
    def getFhirResource(self) -> Condition:
        return super().getFhirResource()

    def get_id(self) -> int:
        return self.getFhirResource().id
    
    def set_id(self, id: str) -> None:
        self.getFhirResource().id = id

    def get_code(self) -> Coding:
        if self.getFhirResource().code:
            for coding in self.getFhirResource().code.coding:
                if coding.system == self.DEFAULT_CODING_SYSTEM and coding.version == self.DEFAULT_CODING_VERSION:
                    return coding
        return None
    
    def set_code(self, coding: Coding) -> None:
        if coding.system is None:
            coding.system = self.DEFAULT_CODING_SYSTEM
        if coding.version is None:
            coding.version = self.DEFAULT_CODING_VERSION
        ext = Extension()
        ext.url = self.OTHER_REASON_URL
        coding.extension = [ext]
        self.getFhirResource().code = CodeableConcept()
        self.getFhirResource().code.coding = [coding]

    def get_category(self) -> Coding:
        if self.getFhirResource().category:
            for coding in self.getFhirResource().category.coding:
                if coding.system == self.DEFAULT_CODING_SYSTEM and coding.version == self.DEFAULT_CODING_VERSION:
                    return coding
        return None
    def set_category(self, coding: Coding) -> None:
        if coding.system is None:
            coding.system = self.DEFAULT_CODING_SYSTEM
        if coding.version is None:
            coding.version = self.DEFAULT_CODING_VERSION
        self.getFhirResource().category = CodeableConcept()
        self.getFhirResource().category.coding = [coding]

    def get_agent(self) -> Coding:
        return self.__get_extension_coding(url=self.AGENT_URL, system=self.DEFAULT_CODING_SYSTEM, version=self.DEFAULT_CODING_VERSION)

    def set_agent(self, coding: Coding) -> None:
        if coding.system is None:
            coding.system = self.DEFAULT_CODING_SYSTEM
        if coding.version is None:
            coding.version = self.DEFAULT_CODING_VERSION
        extension = Extension()
        extension.url = self.AGENT_URL
        extension.valueCodeableConcept = CodeableConcept()
        extension.valueCodeableConcept.coding = [coding]
        self.getFhirResource().extension = self.add_or_update_extension_to_extensions(
            extension=extension,
            extensions=self.getFhirResource().extension)

    def get_antigen(self) -> Coding:
        return self.__get_extension_coding(url=self.ANTIGEN_URL, system=self.DEFAULT_CODING_SYSTEM, version=self.DEFAULT_CODING_VERSION)

    def get_location(self) -> str:
        if self.getFhirResource().extension:
            for extension in self.getFhirResource().extension:
                if extension.url == self.LOCATION_URL and extension.valueString:
                    return extension.valueString
        return None
    
    def set_location(self, location_id: str) -> None:
        extension = Extension()
        extension.url = self.LOCATION_URL
        extension.valueString = location_id
        self.getFhirResource().extension = self.add_or_update_extension_to_extensions(
            extension=extension,
            extensions=self.getFhirResource().extension)
    
    def get_patient_id(self) -> str:
        return self.getFhirResource().patient.reference
    
    def set_patient_id(self, id: str) -> None:
        ref = Reference()
        ref.reference = id
        self.set_patient(patient_ref=ref)

    def get_patient(self) -> Reference:
        return self.getFhirResource().patient
    
    def set_patient(self, patient_ref: Reference) -> None:
        self.getFhirResource().patient = patient_ref

    def __get_extension_coding(self, url, system, version) -> Coding:
        if self.getFhirResource().extension is not None:
            extension: Extension
            coding: Coding
            for extension in self.getFhirResource().extension:
                if extension.url == url and extension.valueCodeableConcept is not None:
                    for coding in extension.valueCodeableConcept.coding:
                        if coding.system == system and coding.version == version:
                            return coding
        return None
    
    def get_evidence(self) -> Coding:
        if self.getFhirResource().evidence is not None and len(self.getFhirResource().evidence) > 0:
            for coding in self.getFhirResource().evidence[0].code.coding:
                if coding.system == self.DEFAULT_CODING_SYSTEM and coding.version == self.DEFAULT_CODING_VERSION:
                    return coding
        return []

    def set_evidence(self, coding: Coding) -> None:
        if self.getFhirResource().evidence is None or len(self.getFhirResource().evidence) == 0:
            self.getFhirResource().evidence = [ConditionEvidence()]
        if coding.system is None:
            coding.system = self.DEFAULT_CODING_SYSTEM
        if coding.version is None:
            coding.version = self.DEFAULT_CODING_VERSION
        codeable_concept = CodeableConcept()
        codeable_concept.coding = [coding]
        self.getFhirResource().evidence[0].code = codeable_concept

    def get_onset_period(self) -> Period:
        return self.getFhirResource().onsetPeriod

    def get_onset_period_start_date(self) -> str:
        return self.fhir_date_to_str_date(self.get_onset_period().start)
    
    def get_onset_period_end_date(self) -> str:
        return self.fhir_date_to_str_date(self.get_onset_period().end)
    
    def set_onset_period(self, period: Period) -> None:
        self.getFhirResource().onsetPeriod = period

    def set_onset_period_from_str(self, start_date: str, end_date: str) -> None:
        period = Period()
        start = self.str_date_to_fhir_date(start_date)
        period.start = start
        end = self.str_date_to_fhir_date(end_date)
        period.end = end
        self.set_onset_period(period)


import logging
from inspqcommun.fhir.clients.base_client import BaseClient
from fhirclient.models.condition import Condition
from requests import Response
import requests
import json

log = logging.getLogger(__name__)
class ConditionClient(BaseClient):
    condition_endpoint = "{base_url}/Condition"
    condition_id_endpoint = "{base_url}/Condition/{id}"
    def __init__(self, base_url=None, base_uri=None, token_header=None, validate_certs=True) -> None:
        super().__init__(base_url=base_url, base_uri=base_uri, token_header=token_header)
        self.validate_certs = validate_certs


    def get_by_id(self, condition_id: int) -> Response:
        response = requests.get(
            url=self.condition_id_endpoint.format(base_url=self.get_fhir_url(), id=condition_id),
            headers=self.headers,
            verify=self.validate_certs
        )
        log.info("GET Condition : {}".format(response.status_code))
        return response

    def create(self, condition: Condition) -> Response:
        data = json.dumps(condition.as_json())
        response = requests.post(
            url=self.condition_endpoint.format(base_url=self.get_fhir_url()),
            data=data,
            headers=self.headers,
            verify=self.validate_certs
        )
        log.info("POST Condition : {}".format(response.status_code))
        return response

    def update(self, condition_id, condition: Condition) -> Response:
        response = requests.put(
            url=self.condition_id_endpoint.format(base_url=self.get_fhir_url(), id=condition_id),
            data=json.dumps(condition.as_json()),
            headers=self.headers,
            verify=self.validate_certs
        )
        log.info("PUT Condition : {}".format(response.status_code))
        return response
    
    def extract_condition_from_response(self, response: Response) -> Condition:
        content = response.json()
        if 'resourceType' in content and content['resourceType'] == 'Condition':
            return Condition(jsondict=content)
        if content['resourceType'] == 'Bundle':
            for entry in content['entry']:
                if 'resource' in entry and entry['resource']['resourceType'] == 'Condition':
                    return Condition(jsondict=entry['resource'])


from inspqcommun.fhir.visitors.base import BaseVisitor
from fhirclient.models.immunizationrecommendation import ImmunizationRecommendation, ImmunizationRecommendationRecommendation
from fhirclient.models.fhirreference import FHIRReference
from fhirclient.models.coding import Coding

class ImmunizationRecommendationVisitor(BaseVisitor):
    def __init__(self, fhir_resource: ImmunizationRecommendation=None) -> None:
        self.setFhirResource(fhir_resource if fhir_resource else ImmunizationRecommendation())

    def getFhirResource(self) -> ImmunizationRecommendation:
        return super().getFhirResource()
    
    def get_id(self) -> str:
        return self.getFhirResource().id
    
    def set_id(self, id: str) -> None:
        self.getFhirResource().id = id

    def get_patient_id(self) -> str:
        return self.getFhirResource().patient.id
    
    def set_patient_id(self, id: str) -> None:
        ref = FHIRReference()
        ref.id = id
        self.getFhirResource().patient = ref

    def get_recommendations(self)-> list[ImmunizationRecommendationRecommendation]:
        return self.getFhirResource().recommendation
    
    def get_recommendation_vaccine_code(self, index: int=0) -> Coding:
        for coding in self.getFhirResource().recommendation[index].vaccineCode.coding:
            if coding.system == self.DEFAULT_CODING_SYSTEM and coding.version == self.DEFAULT_CODING_VERSION:
                return coding
        return None
    
    def get_recommendation_forecast_status(self, index: int=0) -> Coding:
        for coding in self.getFhirResource().recommendation[index].forecastStatus.coding:
            if coding.system == self.DEFAULT_CODING_SYSTEM and coding.version == self.DEFAULT_CODING_VERSION:
                return coding
        return None